/*
 * Solves travelling salesman problem (TSP) with complete, random, and greedy search
 */
 
import java.io.*;
import java.util.*;

public class TSP {
	private final static int INF = Integer.MAX_VALUE; // infinite distance
	private int numNodes;				  // number of nodes in graph
	private int[][] distance;                         // distance matrix
	
	// Calculate shortest path for arbitrary graphs (not suitable for very large graphs!)
	// Returns shortest path as permutation with sequence of node indices
	public Permutation shortestPath() {
		Permutation spath = null;
		int minLength = INF;
		// Use generator to determine all possible permutations of node indices
		PermutationGenerator gen = new PermutationGenerator();
		Set permutations = gen.generate(numNodes);
		Iterator it = permutations.iterator();
		while (it.hasNext()) {
			// Compare length of path to current minimal length
			Permutation path = (Permutation) it.next();
			int length = pathLength(path);
			if (length < minLength) {
				minLength = length;
				spath = path;
			}
		}		
		return spath;
	}
	
	// Calculate approximation of shortest path for arbitrary graphs by greedy search 
	// Returns shortest path as permutation with sequence of node indices
	public Permutation shortestPathGreedy() {
		...
	}

	// Calculate a random path for arbitrary graphs 
	// Returns shortest path as permutation with sequence of node indices
	public Permutation shortestPathRandom() {
		...
	}
	
	// Generate random graph with given number of nodes & max. distance between them
	private void generateRandomGraph(int number, int maxDistance) {
		...
	}
	
	// Calculate length for given path represented as permutation of node indices
	private int pathLength(Permutation path) {
		int result = 0;
		for (int i = 0; i < numNodes; i++) {
			// Account for wrap-around from last node to first node in path
			int from = path.get(i);
			int to = path.get((i+1) % numNodes);
			result += distance[from][to];
		}
		return result;
	}
	
	// Test heuristics with random graphs of growing size
	public static void main(String[] args) {
		TSP tsp = new TSP();
		for (int i = 3; i < 10; i++) {
			tsp.generateRandomGraph(i, 10);
			System.out.println("*** Problem size: " + i);
			...
		}
	}
}
